#include <stdio.h>
#include <windows.h>
#define SECURITY_WIN32 1
#include <security.h>
#include <rpc.h>

static HINSTANCE              gLib = NULL;
static PSecurityFunctionTable gFT = NULL;

static const char *MapErrorCode(int rc)
{
    switch (rc) {
#define CASE_(_x) case _x: return # _x;
    CASE_(SEC_E_OK)
    CASE_(SEC_I_CONTINUE_NEEDED)
    CASE_(SEC_I_COMPLETE_NEEDED)
    CASE_(SEC_I_COMPLETE_AND_CONTINUE)
    CASE_(SEC_E_INCOMPLETE_MESSAGE)
    CASE_(SEC_I_INCOMPLETE_CREDENTIALS)
    CASE_(SEC_E_INVALID_HANDLE)
    CASE_(SEC_E_TARGET_UNKNOWN)
    CASE_(SEC_E_LOGON_DENIED)
    CASE_(SEC_E_INTERNAL_ERROR)
    CASE_(SEC_E_NO_CREDENTIALS)
    CASE_(SEC_E_NO_AUTHENTICATING_AUTHORITY)
    CASE_(SEC_E_INSUFFICIENT_MEMORY)
    CASE_(SEC_E_INVALID_TOKEN)
#undef CASE_
    }
    return "<unknown>";
}

#define NS_ERROR(str) printf(str "\n")

int main()
{
    int rc;
    PSecPkgInfo pkgInfo;
    PSecurityFunctionTable (*initFun)(void);

    gLib = LoadLibrary("secur32.dll");
    if (!gLib) {
        NS_ERROR("failed to load secur32.dll");
        return -1;
    }

    initFun = (PSecurityFunctionTable (*)(void)) GetProcAddress(gLib, "InitSecurityInterfaceA");
    if (!initFun) {
        NS_ERROR("failed to locate DLL entry point");
        return -1;
    }

    gFT = initFun();
    if (!gFT) {
        NS_ERROR("no function table");
        return -1;
    }
    printf("InitSecurityInterfaceA:\n");
    printf("  dwVersion = 0x%x\n", gFT->dwVersion);
    printf("  AcquireCredentialsHandle = %p\n", gFT->AcquireCredentialsHandle);
    printf("  FreeCredentialHandle = %p\n", gFT->FreeCredentialHandle);
    printf("  InitializeSecurityContext = %p\n", gFT->InitializeSecurityContext);
    printf("  AcceptSecurityContext = %p\n", gFT->AcceptSecurityContext);
    printf("  DeleteSecurityContext = %p\n", gFT->DeleteSecurityContext);

    rc = (gFT->QuerySecurityPackageInfo)("NTLM", &pkgInfo);
    if (rc != SEC_E_OK) {
        NS_ERROR("NTLM security package not found");
        return -1;
    }
    printf("QuerySecurityPackageInfo:\n");
    printf("  fCapabilities = 0x%x\n", pkgInfo->fCapabilities);
    printf("  wVersion = 0x%hx\n", pkgInfo->wVersion);
    printf("  wRPCID = 0x%hx\n", pkgInfo->wRPCID);
    printf("  cbMaxToken = %u\n", pkgInfo->cbMaxToken);
    printf("  Name = %s\n", pkgInfo->Name);
    printf("  Comment = %s\n", pkgInfo->Comment);

    return 0;
}
